/*
   interpreter.h - version 1.11, 2002

   cInterpreter is a class that interprets a character string
   with a mathematical expression.

   Copyright (c) 2001-2002 Andreas Jnsson

   This software is provided 'as-is', without any form of
   warranty. In no case will the author be held responsible
   for any damage caused by its use.

   Permission is granted to anyone to use the software for
   for any purpose, including commercial. It is also allowed
   to modify the software and redistribute it free of charge.
   The permission is granted with the following restrictions:

   1. The origin of the software may not be misrepresented.
      It must be plainly understandable who is the author of
      the original software.
   2. Altered versions must not be misrepresented as the
      original software, i.e. must be plainly marked as
      altered.
   3. This notice may not be removed or altered from any
      distribution of the software, altered or not.

   Andreas Jnsson
   andreas@angelcode.com
*/

#ifndef INTERPRETER_H
#define INTERPRETER_H

#include "memory.h"

class cInterpreter
{
public:
	cInterpreter();

	void InterpretLine(const char *line, char *output);

protected:
	char *EatWhites(const char *line);

	// EXPRESSION = TERM [{+|-} TERM]+
	// TERM       = FACTOR [{*|/} FACTOR]+
	// FACTOR     = [-]POWER
	// POWER      = NUMBER[^NUMBER]
	// NUMBER     = ( EXPRESSION ) | [digits][.digits][{e|E}[-|+]digits] | FUNCTION
	// FUNCTION   = name[( EXPRESSION[ , EXPRESSION]+ )]

	char *GetExpression(const char *line, double *pOutput);
	char *GetTerm(const char *line, double *pOutput);
	char *GetFactor(const char *line, double *pOutput);
	char *GetPower(const char *line, double *pOutput);
	char *GetNumber(const char *line, double *pNumber);
	char *GetFunction(const char *line, double *pOutput);
	char *GetName(const char *line, char *name);
	char *GetArguments(const char *line, int numArgs, double *pArgs);

	double lastAnswer;

	cMemory memory;
};

class cCalcException
{
public:
	cCalcException(char *msg) : message(msg) {};
	char *message;
};


#endif